import java.net.URL;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Scanner;

public class Test { 
  private static Scanner scannerFromFilename(String filename) {
    URL url = Test.class.getClassLoader().getResource(filename);
    if (url == null) 
      throw new RuntimeException("File not found: " + filename);
    try {
      Scanner s = new Scanner(url.openStream());
      return s;
    } catch (Exception e) {
      throw new RuntimeException(e + filename + 
      " couldn't be openStreamed");
    }
  }

  private static void assertTrue(boolean b) {
    if (!b) throw new RuntimeException("test failed.");
  }

  private static Vertex findVertex(Collection c, String s) {
    for(Iterator i = c.iterator(); i.hasNext(); ) {
      Vertex v = (Vertex) i.next();
      if (s.equals(v.getLabel())) 
        return v;
    }
    return null;
  }

  public static void main(String[] args) {
    AdjListGraph g = new AdjListGraph(scannerFromFilename("vertex.txt"),
                                      scannerFromFilename("edge.txt"));
    Collection cv = g.vertices();
    Collection ce = g.edges();
    Vertex v = findVertex(cv,"SEA");
    Collection ie = g.incidentEdges(v);
    assertTrue(ie.size()==2);
    String[] arr = new String[2];
    int k = 0;
    for(Iterator i = ie.iterator(); i.hasNext(); ) {
      Edge e = (Edge) i.next();
      arr[k] = e.opposite(v).getLabel();
      k++;
    }
    assertTrue(arr[0].equals("DEN") && arr[1].equals("SFO") ||
               arr[1].equals("DEN") && arr[0].equals("SFO"));        
    assertTrue(g.isAdjacent(findVertex(cv,"ORD"),findVertex(cv,"JFK")));
    assertTrue(g.isAdjacent(findVertex(cv,"JFK"),findVertex(cv,"ORD")));
    assertTrue(!g.isAdjacent(findVertex(cv,"SEA"),findVertex(cv,"JFK")));

    PriorityQueue pq = new PriorityQueue();
    assertTrue(pq.size()==0);
    pq.insert("foo",4);
    pq.insert("bar",2);
    pq.insert("baz",3);
    assertTrue(pq.size()==3);
    assertTrue(pq.min().equals("bar"));
    assertTrue(pq.min().equals("bar"));
    assertTrue(pq.size()==3);
    assertTrue(pq.removeMin().equals("bar"));
    assertTrue(pq.size()==2);
    assertTrue(pq.min().equals("baz"));
    pq.replaceKey("baz",100);
    assertTrue(pq.min().equals("foo"));    
    assertTrue(pq.removeMin().equals("foo"));
    pq.insert("blarg",50);
    assertTrue(pq.removeMin().equals("blarg"));
    assertTrue(pq.removeMin().equals("baz"));
    assertTrue(pq.size()==0);

    GraphAlgorithms ga = new GraphAlgorithms(g);
    ga.computeDijkstras(findVertex(cv,"SEA"));
    // insert your Dijkstras tests here
    ga.computeMST(findVertex(cv,"SEA"));
    // instert your MST tests here
  }

}
